SET ANSI_WARNINGS OFF
GO
SET NOCOUNT ON
GO
USE master
GO
IF DB_ID('MSH2') IS NOT NULL DROP DATABASE MSH2
GO
CREATE DATABASE MSH2
GO
ALTER DATABASE [MSH2] SET ANSI_NULL_DEFAULT OFF
ALTER DATABASE [MSH2] SET ANSI_NULLS OFF
ALTER DATABASE [MSH2] SET ANSI_PADDING OFF
ALTER DATABASE [MSH2] SET ANSI_WARNINGS OFF
ALTER DATABASE [MSH2] SET ARITHABORT OFF
ALTER DATABASE [MSH2] SET AUTO_CLOSE ON
ALTER DATABASE [MSH2] SET AUTO_CREATE_STATISTICS ON
ALTER DATABASE [MSH2] SET AUTO_SHRINK OFF
ALTER DATABASE [MSH2] SET AUTO_UPDATE_STATISTICS ON
ALTER DATABASE [MSH2] SET CURSOR_CLOSE_ON_COMMIT OFF
ALTER DATABASE [MSH2] SET CURSOR_DEFAULT  GLOBAL
ALTER DATABASE [MSH2] SET CONCAT_NULL_YIELDS_NULL OFF
ALTER DATABASE [MSH2] SET NUMERIC_ROUNDABORT OFF
ALTER DATABASE [MSH2] SET QUOTED_IDENTIFIER OFF
ALTER DATABASE [MSH2] SET RECURSIVE_TRIGGERS OFF
ALTER DATABASE [MSH2] SET DISABLE_BROKER
ALTER DATABASE [MSH2] SET AUTO_UPDATE_STATISTICS_ASYNC OFF
ALTER DATABASE [MSH2] SET DATE_CORRELATION_OPTIMIZATION OFF
ALTER DATABASE [MSH2] SET TRUSTWORTHY OFF
ALTER DATABASE [MSH2] SET ALLOW_SNAPSHOT_ISOLATION OFF
ALTER DATABASE [MSH2] SET PARAMETERIZATION SIMPLE
ALTER DATABASE [MSH2] SET READ_WRITE
ALTER DATABASE [MSH2] SET RECOVERY SIMPLE
ALTER DATABASE [MSH2] SET MULTI_USER
ALTER DATABASE [MSH2] SET PAGE_VERIFY CHECKSUM
ALTER DATABASE [MSH2] SET DB_CHAINING OFF
GO

USE [MSH2]
GO

/****** Object:  StoredProcedure [dbo].[ErrorSent]    Script Date: 05/26/2008 16:26:42 ******/
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER OFF
GO
CREATE PROCEDURE [dbo].[ErrorSent]
		@messageId nvarchar(100),
		@errorDescription nvarchar(MAX)
	AS
	
	SET NOCOUNT ON
	
	UPDATE InBox
	SET hasErrors = 1, errorDescription = @errorDescription
	WHERE messageID = @messageId
	
	IF @@ROWCOUNT = 1 
		RETURN 0
	ELSE
		RETURN 1
GO
/****** Object:  StoredProcedure [dbo].[AckSent]    Script Date: 05/26/2008 16:26:42 ******/
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER OFF
GO
CREATE PROCEDURE [dbo].[AckSent]
		@messageId nvarchar(100)
	AS
	
	SET NOCOUNT ON
	
	UPDATE inBox
	SET ackSent = 1
	WHERE messageID = @messageId
	
	IF @@ROWCOUNT = 1 
		RETURN 0
	ELSE
		RETURN 1
GO
/****** Object:  StoredProcedure [dbo].[ErrorReceived]    Script Date: 05/26/2008 16:26:42 ******/
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER OFF
GO
CREATE PROCEDURE [dbo].[ErrorReceived]
		@messageId nvarchar(100),
		@errorDescription nvarchar(MAX)
	AS
	/*
	Viene usata quando si riceve un ErrorMessage relativo ad un messaggio inviato
	*/
	
	SET NOCOUNT ON
	
	UPDATE OutBox
	SET hasErrors = 1, errorDescription = @errorDescription
	WHERE messageID = @messageId
	
	IF @@ROWCOUNT = 1 
		RETURN 0
	ELSE
		RETURN 1
GO
/****** Object:  Table [dbo].[InBox]    Script Date: 05/26/2008 16:26:50 ******/
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
CREATE TABLE [dbo].[InBox](
	[ID] [int] IDENTITY(1,1) NOT NULL,
	[messageId] [nvarchar](100) NOT NULL,
	[conversationId] [nvarchar](100) NOT NULL,
	[timeStamp] [datetime] NOT NULL,
	[addressFrom] [nvarchar](100) NOT NULL,
	[addressTo] [nvarchar](100) NOT NULL,
	[partyIdFrom] [nvarchar](100) NOT NULL,
	[partyIdTo] [nvarchar](100) NOT NULL,
	[SOAPAction] [nvarchar](100) NOT NULL,
	[SOAPStart] [nvarchar](100) NOT NULL,
	[SOAPMessage] [xml] NOT NULL,
	[SOAPPayload] [xml] NOT NULL,
	[PayloadDescription] [nvarchar](100) NOT NULL,
	[EBMSService] [nvarchar](100) NOT NULL,
	[transport] [nvarchar](100) NOT NULL,
	[hasErrors] [smallint] NOT NULL,
	[errorDescription] [nvarchar](max) NOT NULL,
	[ackSent] [smallint] NULL,
	[signed] [smallint] NOT NULL,
	[encrypted] [smallint] NOT NULL,
	[deleted] [smallint] NOT NULL,
	[source] [nvarchar](max) NOT NULL,
 CONSTRAINT [PK_InBox] PRIMARY KEY CLUSTERED 
(
	[ID] ASC
)WITH (PAD_INDEX  = OFF, STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, ALLOW_ROW_LOCKS  = ON, ALLOW_PAGE_LOCKS  = ON) ON [PRIMARY]
) ON [PRIMARY]
GO
/****** Object:  StoredProcedure [dbo].[CheckAckWaiting]    Script Date: 05/26/2008 16:26:42 ******/
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
CREATE PROCEDURE [dbo].[CheckAckWaiting]
--@sendList as
-- Ritorna una stringa con l'elenco dei messageNumber da rispedire
-- il separatore tra i numeri  il pipe (|)
AS

SET NOCOUNT ON

-- Se non ci sono messaggi da inviare di nuovo allora esce
IF (SELECT COUNT(*) FROM WaitForAck WHERE nextSend <= GETDATE()) = 0 RETURN -1

DECLARE @list varchar(100)

SELECT @list = COALESCE(@list + '|', '') + 
   CAST(messageNumber AS nvarchar)
FROM WaitForAck
WHERE nextSend <= GETDATE()

SELECT @list
GO
/****** Object:  Table [dbo].[WaitForAck]    Script Date: 05/26/2008 16:26:59 ******/
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
CREATE TABLE [dbo].[WaitForAck](
	[messageNumber] [int] NOT NULL,
	[messageId] [nvarchar](100) NOT NULL,
	[nextSend] [smalldatetime] NOT NULL,
	[retries] [int] NOT NULL,
	[tried] [int] NOT NULL,
	[retryMinutes] [int] NOT NULL
) ON [PRIMARY]
GO
/****** Object:  Table [dbo].[OutBox]    Script Date: 05/26/2008 16:26:57 ******/
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
CREATE TABLE [dbo].[OutBox](
	[ID] [int] IDENTITY(1,1) NOT NULL,
	[sourceFilename] [nvarchar](200) NOT NULL,
	[messageId] [nvarchar](100) NOT NULL,
	[conversationId] [nvarchar](100) NOT NULL,
	[timeStamp] [datetime] NOT NULL,
	[addressFrom] [nvarchar](100) NOT NULL,
	[addressTo] [nvarchar](100) NOT NULL,
	[partyIdFrom] [nvarchar](100) NOT NULL,
	[partyIdTo] [nvarchar](100) NOT NULL,
	[SOAPAction] [nvarchar](100) NOT NULL,
	[SOAPStart] [nvarchar](100) NOT NULL,
	[SOAPMessage] [xml] NOT NULL,
	[SOAPPayload] [xml] NOT NULL,
	[PayloadDescription] [nvarchar](100) NOT NULL,
	[EBMSService] [nvarchar](100) NOT NULL,
	[transport] [nvarchar](100) NOT NULL,
	[hasErrors] [smallint] NOT NULL,
	[errorDescription] [nvarchar](max) NOT NULL,
	[ackReceived] [smallint] NULL,
	[signed] [smallint] NOT NULL,
	[encrypted] [smallint] NOT NULL,
	[deleted] [smallint] NOT NULL,
 CONSTRAINT [PK_OutBox] PRIMARY KEY CLUSTERED 
(
	[ID] ASC
)WITH (PAD_INDEX  = OFF, STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, ALLOW_ROW_LOCKS  = ON, ALLOW_PAGE_LOCKS  = ON) ON [PRIMARY]
) ON [PRIMARY]
GO
/****** Object:  Table [dbo].[Logs]    Script Date: 05/26/2008 16:26:51 ******/
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
CREATE TABLE [dbo].[Logs](
	[logID] [int] IDENTITY(1,1) NOT NULL,
	[dateTime] [datetime] NOT NULL,
	[entryType] [nvarchar](50) NOT NULL,
	[caller] [nvarchar](50) NOT NULL,
	[message] [nvarchar](4000) NOT NULL,
 CONSTRAINT [PK_log] PRIMARY KEY CLUSTERED 
(
	[logID] ASC
)WITH (PAD_INDEX  = OFF, STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, ALLOW_ROW_LOCKS  = ON, ALLOW_PAGE_LOCKS  = ON) ON [PRIMARY]
) ON [PRIMARY]
GO
/****** Object:  StoredProcedure [dbo].[RemoveAckWaiting]    Script Date: 05/26/2008 16:26:43 ******/
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
CREATE PROCEDURE [dbo].[RemoveAckWaiting]
	@messageId nvarchar(100)
AS

SET NOCOUNT ON

-- L'invio  fallito quindi diminuisco di uno il numero delle rispedizioni
UPDATE WaitForAck SET tried = tried - 1 WHERE messageId = @messageId

-- Questo numero viene inizializzato a zero per cui:
-- .se era un primo invio va a finire a -1 e va cancellato completamente
-- .se era un secondo invio (o pi) va a finire a 0 e mi sta bene

IF (SELECT tried FROM WaitForAck WHERE messageId = @messageId) < 0 BEGIN
	DELETE FROM WaitForAck WHERE messageId = @messageId
END
/****** Object:  StoredProcedure [dbo].[SetMessageWaitingForAck]    Script Date: 05/26/2008 16:26:43 ******/
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
CREATE PROCEDURE [dbo].[SetMessageWaitingForAck]
	@messageNumber INT,
	@messageId nvarchar(100),
	@retries int,
	@retryMinutes int  -- in minuti
AS

SET NOCOUNT ON

-- Se non c' gi una "aspettativa" di Ack in WaitForAck allora la creo
IF (SELECT COUNT(*) FROM WaitForAck WHERE messageId = @messageId) = 0 BEGIN

	-- Calcolo il nextSend
	DECLARE @nextSend AS SMALLDATETIME
	SELECT @nextSend = DATEADD(mi, @retryMinutes, GETDATE())

	INSERT INTO WaitForAck (messageNumber, messageId, nextSend, retries, tried, retryMinutes)
		VALUES (@messageNumber, @messageId, @nextSend, @retries, 0, @retryMinutes)

	END

ELSE BEGIN
-- Se c' gi una "aspettativa" allora incremento di uno il numero dei tentativi fatti

	-- Aggiorno il record nella tabella
	UPDATE WaitForAck 
	SET tried = tried + 1,
		nextSend = DATEADD(mi, retryMinutes, GETDATE())
	WHERE messageId = @messageId

	-- Cancello i record con tried = retries
	DELETE FROM WaitForAck WHERE tried = retries

END
GO
/****** Object:  StoredProcedure [dbo].[GetResendMessageList]    Script Date: 05/26/2008 16:26:43 ******/
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
CREATE PROCEDURE [dbo].[GetResendMessageList]
@sendList AS NVARCHAR(1000) OUTPUT
-- Ritorna la stringa dei messageNumber da rispedire separati dal pipe (|)
AS

SET NOCOUNT ON

SELECT @sendList = COALESCE(@sendList + '|', '') +  CAST(messageNumber AS nvarchar)
FROM WaitForAck
WHERE nextSend <= GETDATE()

IF @sendList IS NULL SET @sendList = ''
GO
/****** Object:  StoredProcedure [dbo].[AckReceived]    Script Date: 05/26/2008 16:26:41 ******/
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER OFF
GO
CREATE PROCEDURE [dbo].[AckReceived]
	@messageId nvarchar(100)
AS
	
SET NOCOUNT ON

-- Aggiorno la tabella OutBox
UPDATE OutBox SET ackReceived = 1 WHERE messageID = @messageId

-- Cancello il messaggio dalla lista di quelli in attesa di Ack
DELETE FROM WaitForAck WHERE messageID = @messageId
GO
/****** Object:  StoredProcedure [dbo].[InsertSentboxPlaceholder]    Script Date: 05/26/2008 16:26:43 ******/
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
CREATE PROCEDURE [dbo].[InsertSentboxPlaceholder]
AS
BEGIN
	SET NOCOUNT ON

	--Cancello cos ne tengo uno alla volta
	DELETE FROM OutBox WHERE messageId = 'dummy'

	INSERT INTO OutBox (messageId, sourceFilename, conversationId, timeStamp, addressFrom, addressTo, partyIdFrom, partyIdTo, SOAPAction, SOAPStart, SOAPMessage, SOAPPayload, PayloadDescription, EBMSService, transport, hasErrors, errorDescription, ackReceived, signed, encrypted, deleted)
	VALUES ('dummy', '', '', '', '', '', '', '', '', '', '', '', '', '', '', 0, '', 0, 0, 0, 1)
	
	RETURN Scope_Identity()
END
GO
/****** Object:  StoredProcedure [dbo].[CreateLogEntry]    Script Date: 05/26/2008 16:26:42 ******/
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER OFF
GO
CREATE PROCEDURE [dbo].[CreateLogEntry]
		@entryType nvarchar(50),
		@caller nvarchar(50),
		@message nvarchar(4000)
	AS
	
	SET NOCOUNT ON
		
	DECLARE @now Datetime
	SET @now = GETDATE()
	
	INSERT INTO Logs(dateTime, entryType, caller, message) VALUES (@now, @entryType, @caller, @message)
	
	IF @@ROWCOUNT = 1 
		RETURN 0
	ELSE
		RETURN 1
GO
